==> emod_api-1.30.0.dev0-py3-none-any.whl: emod_api/interventions/ccdl/ccdl2py.py <==
# The purpose of this script is to take a CCDL file and create a near-working Python script from it.

# Specifically this version creates an emodpy-hiv script. 

# Basically work 1 line at-a-time. parse the:
# - When?
# - Where?
# - Who?
# - Why? (the trigger)
# - What? Which is most of the intervention)

"""
WHEN :: WHERE :: WHO :: WHAT

WHEN: <Start_Time>-<End_Time> OR <Start_Time>(x<Repetitions>/<Time_BetweenReps>)

WHERE: AllPlaces OR Node_List

WHO: <Coverage%>/<IP>/<Min_Age>/<Max_Age>/<Sex>

WHAT: <Triggers>-><Intervention_Name1(Payload)>+<Intervention_Name2(Payload)>+...
"""

from emod_api.interventions.ccdl.ccdl import *

debug = False

broadcasters = ["BroadcastEvent", "SimpleHealthSeekingBehavior", "DiagnosticTreatNeg", "SimpleDiagnostic", "HIVPiecewiseByYearAndSexDiagnostic", "HIVRandomChoice", "HIVMuxer", "STIIsPostDebut", "HIVRapidHIVDiagnostic", "HIVARTStagingCD4AgnosticDiagnostic", "HIVSigmoidByYearAndSexDiagnostic", "HIVDelayedIntervention", "HIVDrawBlood" ]

def get_pos_neg_events( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    payload_str = f"Positive_Event='{pos_event}'"
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]
        payload_str += f", Negative_Event='{neg_event}'"
    return payload_str 

def _build_BroadcastEvent( event_name ):
    call_string = f"BroadcastEvent( camp, '{event_name}' )"
    return call_string

def _build_ART():
    call_string = f"art.new_intervention( camp )"
    return call_string

def _build_SHSB( tendency, event_name ):
    call_string = f"SHSB-TBD"
    return call_string

def _build_ARTDropout():
    call_string = f"artdropout.new_intervention( camp )"
    return call_string

def _build_HIVARTStagingByCD4Diagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"artstagingbycd4diag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVARTStagingCD4AgnosticDiagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"artstagingbycd4agnosticdiag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVDrawBlood( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    call_string = f"drawblood.new_diagnostic( camp,Positive_Event='{pos_event}' )"
    return call_string

def _build_HIVMuxer( event_name ):
    call_string = f"HIVMuxer-TBD( camp, {event_name} )"
    return call_string

def _build_HIVPiecewiseByYearAndSexDiagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"yearandsexdiag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVRandomChoice( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = events[1] if len(events)>1 and events[1] != "null" else None
    choices_dict = { pos_event: 0.5, neg_event: 0.5 }
    call_string = f"random.new_diagnostic( camp, choices={choices_dict} )"
    return call_string

def _build_HIVRapidHIVDiagnostic( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"rapiddiag.new_diagnostic( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' )"
    return call_string

def _build_HIVSigmoidByYearAndSexDiagnostic( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"sigmoidyearandsex.new_diagnostic( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' )"
    return call_string

def _build_MaleCircumcision():
    call_string = f"malecirc( camp )"
    return call_string

def _build_ModifyStiCoInfectionStatus( event_name="Ignore" ):
    call_string = f"modcoinf.new_intervention( camp )" # find
    return call_string

def _build_OutbreakIndividual( event_name ):
    call_string = f"seed( camp, {event_name} )" # placeholder
    return call_string

def _build_PMTCT( event_name ):
    call_string = f"pmtct( camp, {event_name} )" # placeholder
    return call_string

def _build_PREP( event_name ):
    call_string = f"prep.new_intervention( camp )"
    return call_string

def _build_PropertyValueChanger( kvp ):
    try:
        if '=' in kvp:
            tpk, tpv = kvp.split( "=" )
        elif ':' in kvp:
            tpk, tpv = kvp.split( ":" )
        call_string = f"emod-api.interventions.common.PropertyValueChanger( camp, Target_Property_Key='{tpk}', Target_Property_Value='{tpv}' )" # placeholder
    except Exception as ex:
        print( f"Got exception {ex} trying to split {kvp} on ':' or '='." )
        return None
    return call_string

def _build_STIIsPostDebut( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"emodpy_hiv.interventions.ispoststidebut( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' ))"
    return call_string

def _build_SeedOubreak( who ):
    return f"{who}-OutbreakSeed-TBD"

def iv_builder( **kwargs ):
    iv = None
    func_name = kwargs.get( "function", None )
    param1 = kwargs.get( "param1", None )
    param2 = kwargs.get( "param2", None )
    if func_name  == "BroadcastEvent":
        iv = _build_BroadcastEvent( event_name=param1 )
    elif func_name  == "AntiretroviralTherapy":
        iv = _build_ART()
    elif func_name  == "ARTDropout":
        iv = _build_ARTDropout()
    elif func_name  == "BirthTriggeredIV":
        pass
    elif func_name  == "HIVARTStagingByCD4Diagnostic":
        iv = _build_HIVARTStagingByCD4Diagnostic( event_name=param1 ) 
    elif func_name  == "HIVARTStagingCD4AgnosticDiagnostic":
        iv = _build_HIVARTStagingCD4AgnosticDiagnostic( event_name=param1 )
    elif func_name  == "HIVDelayedIntervention":
        # delay+>broadcast
        if post_delay_sep in param1:
            delay_tokens = param1.split( post_delay_sep )
            # delay_config = delay_tokens[0]
            for config_idx in range( len( delay_config ) ):
                delay_config[ config_idx ] = float( delay_config[ config_idx ] )
            iv = _build_HIVDelayedIntervention( event_name=delay_tokens[1], delay=delay_config  )
        else:
            errmsg = f"Delay line seems malformed; can't parse as expected. {param1}"
            print( errmsg )
            #raise ValueError( errmsg )
    elif func_name  == "HIVDrawBlood":
        iv = _build_HIVDrawBlood( event_name=param1 )
    elif func_name  == "HIVMuxer":
        iv = _build_HIVMuxer( event_name=param1 )
    elif func_name  == "HIVPiecewiseByYearAndSexDiagnostic":
        iv = _build_HIVPiecewiseByYearAndSexDiagnostic( event_name=param1 )
    elif func_name  == "HIVRandomChoice":
        iv = _build_HIVRandomChoice( event_name=param1 )
    elif func_name  == "HIVRapidHIVDiagnostic":
        iv = _build_HIVRapidHIVDiagnostic( event_name=param1 )
    elif func_name  == "HIVSigmoidByYearAndSexDiagnostic":
        iv = _build_HIVSigmoidByYearAndSexDiagnostic( event_name=param1 )
    elif func_name  == "MaleCircumcision":
        iv = _build_MaleCircumcision()
    elif func_name  == "ModifyStiCoInfectionStatus":
        iv = _build_ModifyStiCoInfectionStatus( event_name=param1 )
    elif func_name  == "OutbreakIndividual+BroadcastEvent":
        iv = _build_OutbreakIndividual( event_name=param1 )
    elif func_name  == "PMTCT":
        iv = _build_PMTCT( event_name=param1 )
    elif func_name  == "PropertyValueChanger":
        iv = _build_PropertyValueChanger( kvp=param1 )
    elif func_name  == "STIIsPostDebut":
        iv = _build_STIIsPostDebut( event_name=param1 )
    elif func_name  == "PREP":
        iv = _build_PREP( event_name=param1 )
    elif func_name  == "OutbreakIndividual":
        iv = _build_SeedOubreak( who=param1 )
    else:
        raise ValueError( f"Don't know how to build code to create {func_name} intervention." )
        #iv = _build_generic()
    return iv

def _get_who( who_segment ):
    sex = "All"
    tokens = who_segment.split("/")
    coverage = tokens[0].strip( "%" )
    ip = tokens[-1] if "/" in who_segment else "[]"
    ip = ip.replace( ":", "=" )
    min_age = 0
    for tok in tokens:
        if ">" in tok:
            min_age = tok.strip( ">" )
            continue
    max_age = 125
    for tok in tokens:
        if "<" in tok:
            max_age = tok.strip( "<" )
            continue
    if ip == "Male" or ip == "Female":
        sex = ip
    if ">" in ip or "<" in ip or "ale" in ip:
        ip = "[]"
    if debug:
        print( f"coverage={coverage}, sex={sex}, min_age={min_age}, max_age={max_age}, ip={ip}." )
    who = {
        "coverage": float(coverage)*0.01,
        "sex": sex,
        "min_age": min_age,
        "max_age": max_age,
        "ip": ip
    }

    return who

def _get_when( time ):
    # Reject edges where Times don't "overlap"
    if "(" in time:
        time = time.split( "(" )[0]
    time_start = time
    time_end = 1e9
    if "-" in time:
        time_start = float(time.split("-")[0])
        time_end = float(time.split("-")[1])
    if "(" in time:
        time_start = float(time.split("(")[0])
        time_end = float(time.split("(")[1]) 
    time_start = float(time_start)

    return time_start


non_converted_lines = []
def parse( in_name = "campaign.ccdl", out_name = "build_camp.py" ): 

    ccdl = open( in_name ).readlines()

    print( "import emod_api.campaign as camp" )
    print( "import emodpy_hiv.interventions.outbreak as ob " )
    print( "import emodpy_hiv.interventions.cascade_helpers as helpers" )

    for camp_event in ccdl:
        if debug:        
            print( f"Processing line {camp_event}." )

        # We're going to do a brute force purge of DelayedInterventions for now.
        pieces = camp_event.strip().split( main_sep )

        #if "Outbreak" in pieces[-1]:
            # It is useful to 'hack' a few lines so that major infection lifecycle events get tied into
            # our visualization.
            #camp_event = camp_event.replace( pieces[-1], pieces[-1]+"+BroadcastEvent(TBActivation)+BroadcastEvent(NewInfection)+BroadcastEvent(Symptomatic)" )
            #pieces = camp_event.strip().split( main_sep )

        if len( pieces ) < 4:
            error_msg = f"Only found {len(pieces)} in row; '{camp_event}' doesn't look like CCDL." 
            print( error_msg )
            continue
            #raise ValueError( error_msg )

        if "STEERED" in pieces[2]:
            if debug:
                print( "No conversion for ref-tracking." )
            non_converted_lines.append( camp_event )
            continue

        when = pieces[WHEN_IDX]
        where = pieces[WHERE_IDX]
        who = _get_who( pieces[WHO_IDX] )
        if who["coverage"] == 0:
            continue
        what = pieces[WHAT_IDX]


        triggered = what.split( post_trigger_sep )
        why = None
        if len(triggered)>1:
            why = triggered[0].strip()
            if "+" in why:
                why = why.split( "+" ) # multi_trigger_sep?  
            if debug:
                print( f"trigger={why}" )
            what = triggered[-1]

        # In general, Trigger->Delay=>IV1(Payload1)+IV2(Payload2)
        if debug: 
            print( "Basic parsing complete..." )

        from parse import parse
        regex = "{}({})"
        # handle multi-trigger, 
        # and delays
        delay = None
        if post_delay_sep in what:
            # untested code
            delay_split = what.split( post_delay_sep )[-1].strip()
            delay = what.split( post_delay_sep )[0].strip()
            tokens = parse( regex, delay )
            delay = tokens[1].split( "/" )
            for config_idx in range( 1, len( delay ) ):
                delay[ config_idx ] = float( delay[ config_idx ] )
            triggeree = delay_split
        else:
            triggeree = what
        if debug:
            print( f"triggeree={triggeree}" ) 

        ivs = []
        for iv in triggeree.split( multi_iv_sep ):
            # split what into intervention and payload/params.
            # handle params
            sender = iv.strip()
            if debug:
                print( f"'sender' = {sender}" )
            tokens = parse( regex, sender )
            payload = None
            if tokens:
                iv = tokens[0]
                if debug:
                    print( f"IV={iv}" )
                payload = tokens[1]
                if debug:
                    print( f"payload={payload}" )

            iv_code = iv_builder( function=iv, param1=payload )
            print( f"new_iv_{iv} = {iv_code}" )
            ivs.append( f"new_iv_{iv}" )

        if why:
            ivs_str = str(ivs).replace( "'", "" )
            print( f"event = triggered_event_common( camp, '{why}', {ivs_str}, coverage = {who['coverage']}, target_sex='{who['sex']}', target_risk='{who['ip']}', delay={delay} )" )
        else: # scheduled
            print( f"event = ScheduledCampaignEvent( camp, {ivs}, coverage = {who['coverage']}, target_sex='{who['sex']}', target_risk='{who['ip']}' )" )
            print( f"camp.add( event )" )

        print()

import sys
if __name__ == "__main__":
    camp_file = "campaign.ccdl"
    if len(sys.argv)>1:
        camp_file = sys.argv[1]

    parse( in_name=camp_file )

    print( "*** Did not convert...*** " )
    for event in non_converted_lines:
        print( event )

==> emod_api-1.30.1.dev0-py3-none-any.whl: emod_api/interventions/ccdl/ccdl2py.py <==
# The purpose of this script is to take a CCDL file and create a near-working Python script from it.

# Specifically this version creates an emodpy-hiv script. 

# Basically work 1 line at-a-time. parse the:
# - When?
# - Where?
# - Who?
# - Why? (the trigger)
# - What? Which is most of the intervention)

"""
WHEN :: WHERE :: WHO :: WHAT

WHEN: <Start_Time>-<End_Time> OR <Start_Time>(x<Repetitions>/<Time_BetweenReps>)

WHERE: AllPlaces OR Node_List

WHO: <Coverage%>/<IP>/<Min_Age>/<Max_Age>/<Sex>

WHAT: <Triggers>-><Intervention_Name1(Payload)>+<Intervention_Name2(Payload)>+...
"""

from emod_api.interventions.ccdl.ccdl import *

debug = False

broadcasters = ["BroadcastEvent", "SimpleHealthSeekingBehavior", "DiagnosticTreatNeg", "SimpleDiagnostic", "HIVPiecewiseByYearAndSexDiagnostic", "HIVRandomChoice", "HIVMuxer", "STIIsPostDebut", "HIVRapidHIVDiagnostic", "HIVARTStagingCD4AgnosticDiagnostic", "HIVSigmoidByYearAndSexDiagnostic", "HIVDelayedIntervention", "HIVDrawBlood" ]

def get_pos_neg_events( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    payload_str = f"Positive_Event='{pos_event}'"
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]
        payload_str += f", Negative_Event='{neg_event}'"
    return payload_str 

def _build_BroadcastEvent( event_name ):
    call_string = f"BroadcastEvent( camp, '{event_name}' )"
    return call_string

def _build_ART():
    call_string = f"art.new_intervention( camp )"
    return call_string

def _build_SHSB( tendency, event_name ):
    call_string = f"SHSB-TBD"
    return call_string

def _build_ARTDropout():
    call_string = f"artdropout.new_intervention( camp )"
    return call_string

def _build_HIVARTStagingByCD4Diagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"artstagingbycd4diag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVARTStagingCD4AgnosticDiagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"artstagingbycd4agnosticdiag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVDrawBlood( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    call_string = f"drawblood.new_diagnostic( camp,Positive_Event='{pos_event}' )"
    return call_string

def _build_HIVMuxer( event_name ):
    call_string = f"HIVMuxer-TBD( camp, {event_name} )"
    return call_string

def _build_HIVPiecewiseByYearAndSexDiagnostic( event_name ):
    payload_str = get_pos_neg_events( event_name )
    call_string = f"yearandsexdiag.new_diagnostic( camp, {payload_str} )"
    return call_string

def _build_HIVRandomChoice( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = events[1] if len(events)>1 and events[1] != "null" else None
    choices_dict = { pos_event: 0.5, neg_event: 0.5 }
    call_string = f"random.new_diagnostic( camp, choices={choices_dict} )"
    return call_string

def _build_HIVRapidHIVDiagnostic( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"rapiddiag.new_diagnostic( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' )"
    return call_string

def _build_HIVSigmoidByYearAndSexDiagnostic( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"sigmoidyearandsex.new_diagnostic( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' )"
    return call_string

def _build_MaleCircumcision():
    call_string = f"malecirc( camp )"
    return call_string

def _build_ModifyStiCoInfectionStatus( event_name="Ignore" ):
    call_string = f"modcoinf.new_intervention( camp )" # find
    return call_string

def _build_OutbreakIndividual( event_name ):
    call_string = f"seed( camp, {event_name} )" # placeholder
    return call_string

def _build_PMTCT( event_name ):
    call_string = f"pmtct( camp, {event_name} )" # placeholder
    return call_string

def _build_PREP( event_name ):
    call_string = f"prep.new_intervention( camp )"
    return call_string

def _build_PropertyValueChanger( kvp ):
    try:
        if '=' in kvp:
            tpk, tpv = kvp.split( "=" )
        elif ':' in kvp:
            tpk, tpv = kvp.split( ":" )
        call_string = f"emod-api.interventions.common.PropertyValueChanger( camp, Target_Property_Key='{tpk}', Target_Property_Value='{tpv}' )" # placeholder
    except Exception as ex:
        print( f"Got exception {ex} trying to split {kvp} on ':' or '='." )
        return None
    return call_string

def _build_STIIsPostDebut( event_name ):
    events = event_name.split( "/" )
    pos_event = events[0]
    neg_event = None
    if len(events)>1 and events[1] != "null":
        neg_event = events[1]

    call_string = f"emodpy_hiv.interventions.ispoststidebut( camp, Positive_Event='{pos_event}', Negative_Event='{neg_event}' ))"
    return call_string

def _build_SeedOubreak( who ):
    return f"{who}-OutbreakSeed-TBD"

def iv_builder( **kwargs ):
    iv = None
    func_name = kwargs.get( "function", None )
    param1 = kwargs.get( "param1", None )
    param2 = kwargs.get( "param2", None )
    if func_name  == "BroadcastEvent":
        iv = _build_BroadcastEvent( event_name=param1 )
    elif func_name  == "AntiretroviralTherapy":
        iv = _build_ART()
    elif func_name  == "ARTDropout":
        iv = _build_ARTDropout()
    elif func_name  == "BirthTriggeredIV":
        pass
    elif func_name  == "HIVARTStagingByCD4Diagnostic":
        iv = _build_HIVARTStagingByCD4Diagnostic( event_name=param1 ) 
    elif func_name  == "HIVARTStagingCD4AgnosticDiagnostic":
        iv = _build_HIVARTStagingCD4AgnosticDiagnostic( event_name=param1 )
    elif func_name  == "HIVDelayedIntervention":
        # delay+>broadcast
        if post_delay_sep in param1:
            delay_tokens = param1.split( post_delay_sep )
            # delay_config = delay_tokens[0]
            for config_idx in range( len( delay_config ) ):
                delay_config[ config_idx ] = float( delay_config[ config_idx ] )
            iv = _build_HIVDelayedIntervention( event_name=delay_tokens[1], delay=delay_config  )
        else:
            errmsg = f"Delay line seems malformed; can't parse as expected. {param1}"
            print( errmsg )
            #raise ValueError( errmsg )
    elif func_name  == "HIVDrawBlood":
        iv = _build_HIVDrawBlood( event_name=param1 )
    elif func_name  == "HIVMuxer":
        iv = _build_HIVMuxer( event_name=param1 )
    elif func_name  == "HIVPiecewiseByYearAndSexDiagnostic":
        iv = _build_HIVPiecewiseByYearAndSexDiagnostic( event_name=param1 )
    elif func_name  == "HIVRandomChoice":
        iv = _build_HIVRandomChoice( event_name=param1 )
    elif func_name  == "HIVRapidHIVDiagnostic":
        iv = _build_HIVRapidHIVDiagnostic( event_name=param1 )
    elif func_name  == "HIVSigmoidByYearAndSexDiagnostic":
        iv = _build_HIVSigmoidByYearAndSexDiagnostic( event_name=param1 )
    elif func_name  == "MaleCircumcision":
        iv = _build_MaleCircumcision()
    elif func_name  == "ModifyStiCoInfectionStatus":
        iv = _build_ModifyStiCoInfectionStatus( event_name=param1 )
    elif func_name  == "OutbreakIndividual+BroadcastEvent":
        iv = _build_OutbreakIndividual( event_name=param1 )
    elif func_name  == "PMTCT":
        iv = _build_PMTCT( event_name=param1 )
    elif func_name  == "PropertyValueChanger":
        iv = _build_PropertyValueChanger( kvp=param1 )
    elif func_name  == "STIIsPostDebut":
        iv = _build_STIIsPostDebut( event_name=param1 )
    elif func_name  == "PREP":
        iv = _build_PREP( event_name=param1 )
    elif func_name  == "OutbreakIndividual":
        iv = _build_SeedOubreak( who=param1 )
    else:
        raise ValueError( f"Don't know how to build code to create {func_name} intervention." )
        #iv = _build_generic()
    return iv

def _get_who( who_segment ):
    sex = "All"
    tokens = who_segment.split("/")
    coverage = tokens[0].strip( "%" )
    ip = tokens[-1] if "/" in who_segment else "[]"
    ip = ip.replace( ":", "=" )
    min_age = 0
    for tok in tokens:
        if ">" in tok:
            min_age = tok.strip( ">" )
            continue
    max_age = 125
    for tok in tokens:
        if "<" in tok:
            max_age = tok.strip( "<" )
            continue
    if ip == "Male" or ip == "Female":
        sex = ip
    if ">" in ip or "<" in ip or "ale" in ip:
        ip = "[]"
    if debug:
        print( f"coverage={coverage}, sex={sex}, min_age={min_age}, max_age={max_age}, ip={ip}." )
    who = {
        "coverage": float(coverage)*0.01,
        "sex": sex,
        "min_age": min_age,
        "max_age": max_age,
        "ip": ip
    }

    return who

def _get_when( time ):
    # Reject edges where Times don't "overlap"
    if "(" in time:
        time = time.split( "(" )[0]
    time_start = time
    time_end = 1e9
    if "-" in time:
        time_start = float(time.split("-")[0])
        time_end = float(time.split("-")[1])
    if "(" in time:
        time_start = float(time.split("(")[0])
        time_end = float(time.split("(")[1]) 
    time_start = float(time_start)

    return time_start


non_converted_lines = []
def parse( in_name = "campaign.ccdl", out_name = "build_camp.py" ): 

    ccdl = open( in_name ).readlines()

    print( "import emod_api.campaign as camp" )
    print( "import emodpy_hiv.interventions.outbreak as ob " )
    print( "import emodpy_hiv.interventions.cascade_helpers as helpers" )

    for camp_event in ccdl:
        if debug:        
            print( f"Processing line {camp_event}." )

        # We're going to do a brute force purge of DelayedInterventions for now.
        pieces = camp_event.strip().split( main_sep )

        #if "Outbreak" in pieces[-1]:
            # It is useful to 'hack' a few lines so that major infection lifecycle events get tied into
            # our visualization.
            #camp_event = camp_event.replace( pieces[-1], pieces[-1]+"+BroadcastEvent(TBActivation)+BroadcastEvent(NewInfection)+BroadcastEvent(Symptomatic)" )
            #pieces = camp_event.strip().split( main_sep )

        if len( pieces ) < 4:
            error_msg = f"Only found {len(pieces)} in row; '{camp_event}' doesn't look like CCDL." 
            print( error_msg )
            continue
            #raise ValueError( error_msg )

        if "STEERED" in pieces[2]:
            if debug:
                print( "No conversion for ref-tracking." )
            non_converted_lines.append( camp_event )
            continue

        when = pieces[WHEN_IDX]
        where = pieces[WHERE_IDX]
        who = _get_who( pieces[WHO_IDX] )
        if who["coverage"] == 0:
            continue
        what = pieces[WHAT_IDX]


        triggered = what.split( post_trigger_sep )
        why = None
        if len(triggered)>1:
            why = triggered[0].strip()
            if "+" in why:
                why = why.split( "+" ) # multi_trigger_sep?  
            if debug:
                print( f"trigger={why}" )
            what = triggered[-1]

        # In general, Trigger->Delay=>IV1(Payload1)+IV2(Payload2)
        if debug: 
            print( "Basic parsing complete..." )

        from parse import parse
        regex = "{}({})"
        # handle multi-trigger, 
        # and delays
        delay = None
        if post_delay_sep in what:
            # untested code
            delay_split = what.split( post_delay_sep )[-1].strip()
            delay = what.split( post_delay_sep )[0].strip()
            tokens = parse( regex, delay )
            delay = tokens[1].split( "/" )
            for config_idx in range( 1, len( delay ) ):
                delay[ config_idx ] = float( delay[ config_idx ] )
            triggeree = delay_split
        else:
            triggeree = what
        if debug:
            print( f"triggeree={triggeree}" ) 

        ivs = []
        for iv in triggeree.split( multi_iv_sep ):
            # split what into intervention and payload/params.
            # handle params
            sender = iv.strip()
            if debug:
                print( f"'sender' = {sender}" )
            tokens = parse( regex, sender )
            payload = None
            if tokens:
                iv = tokens[0]
                if debug:
                    print( f"IV={iv}" )
                payload = tokens[1]
                if debug:
                    print( f"payload={payload}" )

            iv_code = iv_builder( function=iv.strip(), param1=payload )
            print( f"new_iv_{iv} = {iv_code}" )
            ivs.append( f"new_iv_{iv}" )

        if why:
            ivs_str = str(ivs).replace( "'", "" )
            print( f"event = triggered_event_common( camp, '{why}', {ivs_str}, coverage = {who['coverage']}, target_sex='{who['sex']}', target_risk='{who['ip']}', delay={delay} )" )
        else: # scheduled
            print( f"event = ScheduledCampaignEvent( camp, {ivs}, coverage = {who['coverage']}, target_sex='{who['sex']}', target_risk='{who['ip']}' )" )
            print( f"camp.add( event )" )

        print()

import sys
if __name__ == "__main__":
    camp_file = "campaign.ccdl"
    if len(sys.argv)>1:
        camp_file = sys.argv[1]

    parse( in_name=camp_file )

    print( "*** Did not convert...*** " )
    for event in non_converted_lines:
        print( event )
